from demands import JSONServiceClient, HTTPServiceClient


class ServiceRatingApiClient(object):
    """
    Client for Service Rating Service.

    :param str auth_token:
        An access token.

    :param str api_url:
        The full URL of the API.

    """

    def __init__(self, auth_token, api_url, session=None, session_http=None):
        if session is None:
            session = JSONServiceClient(
                url=api_url, headers={'Authorization': 'Token ' + auth_token})

        if session_http is None:
            session_http = HTTPServiceClient(
                url=api_url, headers={'Authorization': 'Token ' + auth_token})
        self.session = session
        self.session_http = session_http

    # Invites
    def get_invites(self, params=None):
        return self.session.get('/invite/', params=params)

    def get_invite(self, invite_id):
        return self.session.get('/invite/%s/' % invite_id)

    def create_invite(self, invite):
        return self.session.post('/invite/', data=invite)

    def update_invite(self, invite_id, data=None):
        return self.session.patch('/invite/%s/' % invite_id, data=data)

    def delete_invite(self, invite_id):
        # Ratings should be deleted first for FK reasons
        self.session.delete('/invite/%s/' % invite_id)
        return {"success": True}

    # Ratings
    def get_ratings(self, params=None):
        return self.session.get('/rating/', params=params)

    def get_rating(self, rating_id):
        return self.session.get('/rating/%s/' % rating_id)

    def create_rating(self, rating):
        return self.session.post('/rating/', data=rating)

    def update_rating(self, rating_id, data=None):
        return self.session.patch('/rating/%s/' % rating_id, data=data)

    def delete_rating(self, rating_id):
        self.session.delete('/rating/%s/' % rating_id)
        return {"success": True}




"""
Client for Messaging Content Store HTTP services APIs.

"""
import requests
import json


class SchedulerApiClient(object):

    """
    Client for Scheduler API.

    :param str api_token:

        An API Token.

    :param str api_url:
        The full URL of the API. Defaults to
        ``http://seed-scheduler/api/v1``.

    """

    def __init__(self, api_token, api_url=None, session=None):
        if api_url is None:
            api_url = "http://seed-scheduler/api/v1"
        self.api_url = api_url
        self.headers = {
            'Content-Type': 'application/json',
            'Authorization': 'Token %s' % api_token
        }
        if session is None:
            session = requests.Session()
        session.headers.update(self.headers)
        self.session = session

    def call(self, endpoint, method, obj=None, params=None, data=None):
        if obj is None:
            url = '%s/%s/' % (self.api_url.rstrip('/'), endpoint)
        else:
            url = '%s/%s/%s/' % (self.api_url.rstrip('/'), endpoint, obj)
        result = {
            'get': self.session.get,
            'post': self.session.post,
            'patch': self.session.patch,
            'delete': self.session.delete,
        }.get(method, None)(url, params=params, data=json.dumps(data))
        result.raise_for_status()
        if method is "delete":  # DELETE returns blank body
            return {"success": True}
        else:
            return result.json()

    def get_schedules(self, params=None):
        return self.call('schedule', 'get', params=params)

    def get_schedule(self, schedule_id):
        return self.call('schedule', 'get', obj=schedule_id)

    def create_schedule(self, schedule):
        return self.call('schedule', 'post', data=schedule)

    def update_schedule(self, schedule_id, schedule):
        return self.call('schedule', 'patch', obj=schedule_id,
                         data=schedule)

    def delete_schedule(self, schedule_id):
        # Schedule messages must all be deleted first for FK reasons
        return self.call('schedule', 'delete', obj=schedule_id)

    def get_failed_tasks(self, params=None):
        return self.call('failed-tasks', 'get', params=params)

    def requeue_failed_tasks(self):
        return self.call('failed-tasks', 'post', data={})



from demands import JSONServiceClient, HTTPServiceClient


class AuthApiClient(object):
    """
    Client for Auth Service.

    :param str email:

        An email address.

    :param str password:

        A password.

    :param str api_url:
        The full URL of the API.

    """

    def __init__(self, email, password, api_url, session=None,
                 session_http=None):
        if session is None:
            session = JSONServiceClient(url=api_url)
        # login
        data = {"email": email, "password": password}
        login = session.post('/user/tokens/', data=data)
        self.token = login["token"]
        headers = {'Authorization': 'Token %s' % self.token}
        session = JSONServiceClient(url=api_url, headers=headers)
        self.session = session
        if session_http is None:
            session_http = HTTPServiceClient(url=api_url, headers=headers)
        self.session_http = session_http

    def get_permissions(self):
        return self.session.get('/user/')

    def get_users(self):
        return self.session.get('/users/')

    def create_user(self, user):
        return self.session.post('/users/', data=user)

    def get_user(self, user):
        return self.session.get('/users/%s/' % user)

    def update_user(self, user_id, user):
        return self.session.put('/users/%s/' % user_id, data=user)

    def remove_user_from_team(self, user, team):
        # Returns a 204 with empty content so lets return True if it worked
        response = self.session_http.delete('/teams/%s/users/%s/' % (
                                            team, user,))
        if response.status_code == 204:
            return True
        else:
            return False

    def add_user_to_team(self, user, team):
        # Returns a 204 with empty content so lets return True if it worked
        response = self.session_http.put('/teams/%s/users/%s/' % (team, user,))
        if response.status_code == 204:
            return True
        else:
            return False

    def delete_user(self, user_id):
        # archives, soft delete
        response = self.session_http.delete('/users/%s/' % user_id)
        if response.status_code == 204:
            return True
        else:
            return False

    def get_teams(self):
        return self.session.get('/teams/')

    def create_team(self, org, team):
        return self.session.post('/organizations/%s/teams/' % org, data=team)

    def create_permission(self, team, permission):
        return self.session.post('/teams/%s/permissions/' % team,
                                 data=permission)



from demands import JSONServiceClient, HTTPServiceClient


class StageBasedMessagingApiClient(object):
    """
    Client for Stage Based Messaging Service.

    :param str auth_token:

        An access token.

    :param str api_url:
        The full URL of the API.

    """

    def __init__(self, auth_token, api_url, session=None, session_http=None):
        if session is None:
            session = JSONServiceClient(
                url=api_url, headers={'Authorization': 'Token ' + auth_token})

        if session_http is None:
            session_http = HTTPServiceClient(
                url=api_url, headers={'Authorization': 'Token ' + auth_token})
        self.session = session
        self.session_http = session_http

    def get_schedules(self, params=None):
        return self.session.get('/schedule/', params=params)

    def get_schedule(self, schedule_id):
        return self.session.get('/schedule/%s/' % schedule_id)

    def get_messagesets(self, params=None):
        return self.session.get('/messageset/', params=params)

    def get_messageset(self, messageset_id):
        return self.session.get('/messageset/%s/' % messageset_id)

    def get_messageset_languages(self):
        return self.session.get('/messageset_languages/')

    def get_subscription(self, subscription):
        return self.session.get('/subscriptions/%s/' % subscription)

    def get_subscriptions(self, params=None):
        return self.session.get('/subscriptions/', params=params)

    def get_messages(self, params=None):
        return self.session.get('/message/', params=params)

    def get_message(self, message_id):
        return self.session.get('/message/%s/' % message_id)

    def delete_message(self, message_id):
        return self.session.delete('/message/%s/' % message_id)

    def delete_binarycontent(self, binarycontent_id):
        return self.session.delete('/binarycontent/%s/' % binarycontent_id)

    def create_message(self, message):
        return self.session.post('/message/', data=message)

    def create_binarycontent(self, content):
        return self.session_http.post('/binarycontent/', files=content).json()

    def update_subscription(self, subscription, data=None):
        return self.session.patch('/subscriptions/%s/' % subscription,
                                  data=data)

    def create_subscription(self, subscription):
        return self.session.post('/subscriptions/', data=subscription)

    def get_failed_tasks(self, params=None):
        return self.session.get('/failed-tasks/', params=params)

    def requeue_failed_tasks(self):
        return self.session.post('/failed-tasks/')



from demands import JSONServiceClient
from requests.adapters import HTTPAdapter


class SeedServicesApiClient(object):
    """
    Base API client for seed services.

    :param str auth_token:
        An access token.

    :param str api_url:
        The full URL of the API.

    :param JSONServiceClient session:
        An instance of JSONServiceClient to use

    :param retries:
        (optional) The number of times to retry an HTTP request

    """

    def __init__(self, auth_token, api_url, session=None, retries=0):
        headers = {'Authorization': 'Token ' + auth_token}
        if session is None:
            session = JSONServiceClient(url=api_url,
                                        headers=headers)
        self.session = session

        if retries > 0:
            http = HTTPAdapter(max_retries=retries)
            https = HTTPAdapter(max_retries=retries)
            self.session.mount('http://', http)
            self.session.mount('https://', https)



from demands import JSONServiceClient


class MetricsApiClient(object):
    """
    Client for the Metrics API.

    :param str url: The full URL of the API
    :param tuple auth:
        A tuple containing (username, password) for the authentication of the
        service. Defaults to not using authentication.
    :param demands.JSONServiceClient session:
        The session to use for requests. Defaults to creating a new session.
    """
    def __init__(self, url, auth=None, session=None):
        if session is None:
            session = JSONServiceClient(
                url=url,
                auth=auth)
        self.session = session

    def get_metrics(self, **kwargs):
        """
        Gets the metrics for the specified parameters.

        :param list_or_str m:
            A list of strings representing the metric names that you want to
            query, or a string representing the metric name of the single
            metric that you want to query.
        :param str from:
            The relative or absolute time to start the time period.
        :param str until:
            The relative or absolute time to end the time period.
        :param str nulls:
            What to do with null values. 'keep' keeps the null values, 'omit'
            removes the null values from the returned data points, 'zeroize'
            converts all null values to zero.
        :param str interval:
            The time length that each data point should cover, ie. the time
            resolution of the returned data.
        :param bool align_to_from:
            If False, the buckets will be aligned according to the nearest
            rounded value, ie. for a 10 minute interval, buckets will start at
            00h00, 00h10, 00h20, etc.
            If True, the buckets will be aligned according to the 'from' value
        """
        if 'from_' in kwargs:
            from_ = kwargs.pop('from_')
            kwargs['from'] = from_

        return self.session.get('/metrics/', params=kwargs)

    def fire_metrics(self, **kwargs):
        """
        Fires data points for a specific metrics.

        Takes in arguments in the format 'metric_name=metric_value', where
        metric_name is the name of the metric that you want to fire a data
        point for, and metric_value is the value you want for fire for that
        metric.
        """
        return self.session.post('/metrics/', data=kwargs)



from .seed_services import SeedServicesApiClient


class IdentityStoreApiClient(SeedServicesApiClient):
    """
    Client for Identity Store Service.

    :param str auth_token:
        An access token.

    :param str api_url:
        The full URL of the API.

    :param JSONServiceClient session:
        An instance of JSONServiceClient to use

    """

    def __init__(self, auth_token, api_url, session=None, **kwargs):
        super(IdentityStoreApiClient, self).__init__(
            auth_token, api_url, session=session, **kwargs)

    def get_identities(self, params=None):
        return self.session.get('/identities/', params=params)

    def search_identities(self, field, value):
        # this is used for searching 'details' field to avoid DRF lacks
        # use "details__preferred_language" for example field
        params = {field: value}
        return self.session.get('/identities/search/', params=params)

    def get_identity(self, identity):
        # return None on 404 becuase that means an identity not found
        result = self.session.get('/identities/%s/' % identity,
                                  expected_response_codes=[404, 200])
        if "detail" in result and result["detail"] == "Not found.":
            return None
        return result

    def get_identity_by_address(self, address_type, address_value):
        params = {"details__addresses__%s" % address_type: address_value}
        return self.session.get('/identities/search/', params=params)

    def get_identity_address(self, identity_id, address_type='msisdn'):
        params = {'default': True}

        response = self.session.get(
            '/identities/{0}/addresses/{1}'.format(identity_id, address_type),
            params=params)

        if len(response["results"]) > 0:
            return response["results"][0]["address"]
        else:
            return None

    def update_identity(self, identity, data=None):
        return self.session.patch('/identities/%s/' % identity, data=data)

    def create_identity(self, identity):
        return self.session.post('/identities/', data=identity)

    def get_optouts(self, params=None):
        return self.session.get('/optouts/search/', params=params)

    def create_optout(self, optout):
        return self.session.post('/optout/', data=optout)

    def create_optin(self, optin):
        return self.session.post('/optin/', data=optin)
